import Echo from "laravel-echo";
import PusherPkg from "pusher-js";
import { createConsola } from "consola";
import { useEchoConfig } from "./composables/useEchoConfig.js";
import { defineNuxtPlugin, createError, useCookie } from "#app";
const Pusher = PusherPkg.default || PusherPkg;
const MODULE_NAME = "nuxt-laravel-echo";
function createEchoLogger(logLevel) {
  return createConsola({ level: logLevel }).withTag(MODULE_NAME);
}
const readCsrfCookie = (name) => useCookie(name, { readonly: true });
function createFetchClient(authentication, logger) {
  const fetchOptions = {
    baseURL: authentication.baseUrl,
    credentials: "include",
    retry: false,
    async onRequest(context) {
      let csrfToken = readCsrfCookie(authentication.csrfCookie);
      if (!csrfToken.value) {
        await $fetch(authentication.csrfEndpoint, {
          baseURL: authentication.baseUrl,
          credentials: "include",
          retry: false
        });
        csrfToken = readCsrfCookie(authentication.csrfCookie);
      }
      if (!csrfToken.value) {
        logger.warn(
          `${authentication.csrfCookie} cookie is missing, unable to set ${authentication.csrfHeader} header`
        );
        return;
      }
      context.options.headers = {
        ...context.options.headers,
        [authentication.csrfHeader]: csrfToken.value
      };
    }
  };
  return $fetch.create(fetchOptions);
}
function createAuthorizer(authentication, logger) {
  const client = createFetchClient(authentication, logger);
  const authorizer = (channel, _) => {
    return {
      authorize: (socketId, callback) => {
        const payload = JSON.stringify({
          socket_id: socketId,
          channel_name: channel.name
        });
        client(authentication.authEndpoint, {
          method: "post",
          body: payload
        }).then(
          (response) => callback(null, response)
        ).catch((error) => callback(error, null));
      }
    };
  };
  return authorizer;
}
function prepareEchoOptions(config, logger) {
  const forceTLS = config.scheme === "https";
  const additionalOptions = config.properties || {};
  const authorizer = config.authentication ? createAuthorizer(
    config.authentication,
    logger
  ) : void 0;
  if (config.broadcaster === "pusher") {
    if (forceTLS === false) {
      throw createError('Pusher requires a secure connection (schema: "https")');
    }
    return {
      broadcaster: config.broadcaster,
      key: config.key,
      cluster: config.cluster,
      forceTLS,
      authorizer,
      ...additionalOptions
    };
  }
  return {
    broadcaster: config.broadcaster,
    key: config.key,
    wsHost: config.host,
    wsPort: config.port,
    wssPort: config.port,
    forceTLS,
    enabledTransports: config.transports,
    authorizer,
    ...additionalOptions
  };
}
export default defineNuxtPlugin((_nuxtApp) => {
  const config = useEchoConfig();
  const logger = createEchoLogger(config.logLevel);
  window.Pusher = Pusher;
  window.Echo = new Echo(prepareEchoOptions(config, logger));
  logger.debug("Laravel Echo client initialized");
  return {
    provide: {
      echo: window.Echo
    }
  };
});
